package com.eftpay.eft_flutter_amap;

import android.app.Activity;
import android.content.Intent;
import android.util.Log;
import android.view.View;

import com.amap.api.location.AMapLocation;
import com.amap.api.location.AMapLocationClient;
import com.amap.api.location.AMapLocationClientOption;
import com.amap.api.location.AMapLocationListener;
import com.amap.api.maps.model.LatLng;
import com.amap.api.maps.model.Poi;
import com.amap.api.navi.AmapNaviPage;
import com.amap.api.navi.AmapNaviParams;
import com.amap.api.navi.AmapNaviType;
import com.amap.api.navi.INaviInfoCallback;
import com.amap.api.navi.model.AMapNaviLocation;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** EftFlutterAmapPlugin */
public class EftFlutterAmapPlugin implements MethodCallHandler {
  Activity activity;
  MethodChannel channel;
  EftFlutterAmapPlugin(Activity activity,MethodChannel channel){
    this.activity = activity;
    this.channel = channel;
  }
  /** Plugin registration. */
  public static void registerWith(Registrar registrar) {
    MethodChannel channel = new MethodChannel(registrar.messenger(), "eft_flutter_amap");
    channel.setMethodCallHandler(new EftFlutterAmapPlugin(registrar.activity(),channel));
  }

  @Override
  public void onMethodCall(MethodCall call, Result result) {
    Log.d("onMethodCall","method="+call.method+"|args="+call.arguments);

    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    } else if (call.method.equals("initKey")) {
      result.success(true);
    } else  if (call.method.equals("initLocation")) {
      boolean onceFlag = false;
      Map<String,String> map = (Map<String, String>) call.arguments;
      onceFlag = "1".equals(map.get("onceFlag"));
      initLocation(onceFlag);

      result.success(true);
    } else  if (call.method.equals("stopLocation")) {
      stopLocation();

      result.success(true);
    } else  if (call.method.equals("destoryLocation")) {
      destoryLocation();

      result.success(true);
    } else  if (call.method.equals("showMap")) {
      //    {
//      latitude = "36.0";
//      longitude = "113.0";
//    }

      String title = "高德地图";
      String marker = "";
      Map<String,String> map = (Map<String, String>) call.arguments;
      title = map.get("title");
      marker = map.get("marker");
      double latitude = Double.parseDouble(map.get("latitude"));
      double longitude = Double.parseDouble(map.get("longitude"));
      Intent it = new Intent(activity,AMapActivity.class);
      it.putExtra("title",title);
      it.putExtra("marker",marker);
      it.putExtra("latitude",latitude);
      it.putExtra("longitude",longitude);
      activity.startActivity(it);

      result.success(true);
    } else if (call.method.equals("showNavMap")) {
      //    {
//      endAddress = "\U7ed3\U675f\U5730\U5740";
//      latitudeEnd = "37.0";
//      latitudeStart = "36.0";
//      longitudeEnd = "114.0";
//      longitudeStart = "113.0";
//      startAddress = "\U5f00\U59cb\U5730\U5740";
//    }
      Map<String,String> map = (Map<String, String>) call.arguments;
      String startAddress = map.get("startAddress");
      double latitudeStart = Double.parseDouble(map.get("latitudeStart"));
      double longitudeStart = Double.parseDouble(map.get("longitudeStart"));
      String endAddress = map.get("endAddress");
      double latitudeEnd = Double.parseDouble(map.get("latitudeEnd"));
      double longitudeEnd = Double.parseDouble(map.get("longitudeEnd"));
      Poi start = new Poi(startAddress, new LatLng(latitudeStart,longitudeStart), "");
/**终点传入的是北京站坐标,但是POI的ID "B000A83M61"对应的是北京西站，所以实际算路以北京西站作为终点**/
      Poi end = new Poi(endAddress, new LatLng(latitudeEnd, longitudeEnd), "");
      AmapNaviPage.getInstance().showRouteActivity(activity, new AmapNaviParams(start, null, end, AmapNaviType.DRIVER), myINaviInfoCallback);


      result.success(true);
    } else {
      result.notImplemented();
    }
  }
  public INaviInfoCallback myINaviInfoCallback = new INaviInfoCallback(){

    @Override
    public void onInitNaviFailure() {

    }

    @Override
    public void onGetNavigationText(String s) {

    }

    @Override
    public void onLocationChange(AMapNaviLocation aMapNaviLocation) {

    }

    @Override
    public void onArriveDestination(boolean b) {

    }

    @Override
    public void onStartNavi(int i) {

    }

    @Override
    public void onCalculateRouteSuccess(int[] ints) {

    }

    @Override
    public void onCalculateRouteFailure(int i) {

    }

    @Override
    public void onStopSpeaking() {

    }

    @Override
    public void onReCalculateRoute(int i) {

    }

    @Override
    public void onExitPage(int i) {

    }

    @Override
    public void onStrategyChanged(int i) {

    }

    @Override
    public View getCustomNaviBottomView() {
      return null;
    }

    @Override
    public View getCustomNaviView() {
      return null;
    }

    @Override
    public void onArrivedWayPoint(int i) {

    }
  };


  //声明AMapLocationClient类对象
  public static AMapLocationClient mLocationClient = null;

  //声明定位回调监听器
  public AMapLocationListener mLocationListener = new AMapLocationListener() {
    @Override
    public void onLocationChanged(AMapLocation aMapLocation) {
      if (aMapLocation != null) {
        if (aMapLocation.getErrorCode() == 0) {
          //可在其中解析amapLocation获取相应内容。
          double latitude = aMapLocation.getLatitude();
          double longitude = aMapLocation.getLongitude();
          HashMap<String,String> map = new HashMap<>();
          map.put("latitude",""+latitude);
          map.put("longitude",""+longitude);
          if(channel!=null){
            channel.invokeMethod("locationUpdate",map);
          }
        }else {
          //定位失败时，可通过ErrCode（错误码）信息来确定失败的原因，errInfo是错误信息，详见错误码表。
          Log.e("AmapError","location Error, ErrCode:"
                  + aMapLocation.getErrorCode() + ", errInfo:"
                  + aMapLocation.getErrorInfo());
        }
      }
    }
  };


  public void initLocation(boolean onceFlag){

    //初始化定位
    if(mLocationClient==null) {
      mLocationClient = new AMapLocationClient(activity);
    }
    //设置定位回调监听
    mLocationClient.setLocationListener(mLocationListener);


    AMapLocationClientOption option = new AMapLocationClientOption();

    //设置定位模式为AMapLocationMode.Hight_Accuracy，高精度模式。
    option.setLocationMode(AMapLocationClientOption.AMapLocationMode.Hight_Accuracy);
    if(onceFlag){
      //获取一次定位结果：
      //该方法默认为false。
      option.setOnceLocation(true);

      //获取最近3s内精度最高的一次定位结果：
      //设置setOnceLocationLatest(boolean b)接口为true，启动定位时SDK会返回最近3s内精度最高的一次定位结果。如果设置其为true，setOnceLocation(boolean b)接口也会被设置为true，反之不会，默认为false。
      option.setOnceLocationLatest(true);
    }else {
      option.setInterval(15000);
    }

    mLocationClient.setLocationOption(option);
    //设置场景模式后最好调用一次stop，再调用start以保证场景模式生效
    mLocationClient.stopLocation();
    mLocationClient.startLocation();

  }
  public void stopLocation(){
    if(mLocationClient!=null){
      mLocationClient.stopLocation();
    }
  }
  public void destoryLocation(){
    if(mLocationClient!=null){
      mLocationClient.onDestroy();
      mLocationClient = null;
    }
  }
}
