#import "EftFlutterAmapPlugin.h"
#import <AMapFoundationKit/AMapFoundationKit.h>
#import <AMapLocationKit/AMapLocationKit.h>
#import <MAMapKit/MAMapKit.h>
#import <AMapNaviKit/AMapNaviKit.h>
#import <CoreLocation/CLLocation.h>
@interface EftFlutterAmapPlugin()<AMapNaviCompositeManagerDelegate,MAMapViewDelegate,AMapLocationManagerDelegate>
@property (nonatomic, strong) AMapLocationManager *locationManager;
@property (nonatomic, strong) AMapNaviCompositeManager *compositeManager;
@property (nonatomic, assign) UIViewController *root;
@property (nonatomic, assign) FlutterMethodChannel *channel;
@property (nonatomic, assign) UIViewController *mapViewController;
@end
@implementation EftFlutterAmapPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"eft_flutter_amap"
            binaryMessenger:[registrar messenger]];
  UIViewController *root = UIApplication.sharedApplication.delegate.window.rootViewController;
  EftFlutterAmapPlugin* instance = [[EftFlutterAmapPlugin alloc] initWithRoot:root channel:channel];
  [instance compositeManager];
  [registrar addMethodCallDelegate:instance channel:channel];
}
- (id)initWithRoot:(UIViewController *)root channel:(FlutterMethodChannel *)channel {
  if (self = [super init]) {
    self.root = root;
    self.channel = channel;
  }
  return self;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  NSLog(@"method=%@,args=%@",call.method,call.arguments);
  if ([@"getPlatformVersion" isEqualToString:call.method]) {
    result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  } else if ([@"initKey" isEqualToString:call.method]) {
    NSString* key = call.arguments;
    [AMapServices sharedServices].apiKey =key;
    result(@YES);
  } else if ([@"initLocation" isEqualToString:call.method]) {
    NSDictionary* dic = call.arguments;
    BOOL onceFlag = [[dic objectForKey:@"onceFlag"]intValue]==1;
    [self initLocation:onceFlag];
    result(@YES);
  } else if ([@"stopLocation" isEqualToString:call.method]) {
    if(self.locationManager!=nil){
      [self.locationManager stopUpdatingLocation];
    }
    
    result(@YES);
  } else if ([@"destoryLocation" isEqualToString:call.method]) {
    if(self.locationManager!=nil){
      [self.locationManager stopUpdatingLocation];
      self.locationManager = nil;
    }
    
    result(@YES);
  } else if ([@"showMap" isEqualToString:call.method]) {
//    {
//      latitude = "36.0";
//      longitude = "113.0";
//    }
    NSDictionary* dic = call.arguments;
    NSString* title = [dic objectForKey:@"title"];
    NSString* marker = [dic objectForKey:@"marker"];
    NSString* latitude = [dic objectForKey:@"latitude"];
    NSString* longitude = [dic objectForKey:@"longitude"];
    
    //将属性映射到view上面
    UIViewController *vc = [[UIViewController alloc] init ];
    /*创建地图并添加到父view上*/
    MAMapView* mapView = [[MAMapView alloc] initWithFrame:vc.view.bounds];
    mapView.showsScale = YES;
    mapView.showsUserLocation = YES;
    
    
    if(latitude&&longitude){
      CLLocationCoordinate2D center = (CLLocationCoordinate2D){
        [latitude doubleValue], [longitude doubleValue]
      };
      mapView.centerCoordinate = center;
    }
    mapView.zoomLevel = 16;
    mapView.delegate = self;
    vc.view = mapView;
    vc.title = @"高德地图";
    self.mapViewController = vc;
    
    UIBarButtonItem* back = [[UIBarButtonItem alloc] initWithTitle:@"返回" style:UIBarButtonItemStyleDone target:self action:@selector(dismiss)];
    vc.navigationItem.leftBarButtonItem = back;
    
    
    UINavigationController *navController = [[UINavigationController alloc]initWithRootViewController:vc];
    [navController.navigationBar setBarTintColor:[UIColor colorWithRed:0 green:0 blue:1 alpha:1]];
  
    // 导航栏背景颜色
    [navController.navigationBar setBarTintColor:[UIColor colorWithRed:0 green:131.0/255.0 blue:242.0/255.0 alpha:1]];
    
    // 导航栏标题字体颜色
    [navController.navigationBar setTitleTextAttributes:@{NSForegroundColorAttributeName:[UIColor whiteColor]}];
    
    // 导航栏左右按钮字体颜色
    navController.navigationBar.tintColor = [UIColor whiteColor];
    
//    [navController setNavigationBarHidden:YES animated:NO];
    
//     navController.navigationBar.translucent = NO;
    [self.root presentViewController:navController animated:true completion:nil];
    
    MAPointAnnotation *pointAnnotation = [[MAPointAnnotation alloc] init];
    pointAnnotation.coordinate = CLLocationCoordinate2DMake(latitude.doubleValue, longitude.doubleValue);
    pointAnnotation.title = title;
    pointAnnotation.subtitle = marker;
    [mapView addAnnotation:pointAnnotation];
    
    result(@YES);
  } else if ([@"showNavMap" isEqualToString:call.method]) {
//    {
//      endAddress = "\U7ed3\U675f\U5730\U5740";
//      latitudeEnd = "37.0";
//      latitudeStart = "36.0";
//      longitudeEnd = "114.0";
//      longitudeStart = "113.0";
//      startAddress = "\U5f00\U59cb\U5730\U5740";
//    }
    NSDictionary* dic = call.arguments;
    NSString* startAddress = [dic objectForKey:@"startAddress"];
    NSString* latitudeStart = [dic objectForKey:@"latitudeStart"];
    NSString* longitudeStart = [dic objectForKey:@"longitudeStart"];
    NSString* endAddress = [dic objectForKey:@"endAddress"];
    NSString* latitudeEnd = [dic objectForKey:@"latitudeEnd"];
    NSString* longitudeEnd = [dic objectForKey:@"longitudeEnd"];
    
    
    AMapNaviCompositeUserConfig *config = [[AMapNaviCompositeUserConfig alloc] init];
    if(startAddress&&latitudeStart&&longitudeStart){
      [config setRoutePlanPOIType:AMapNaviRoutePlanPOITypeStart location:[AMapNaviPoint locationWithLatitude:[latitudeStart doubleValue] longitude:[longitudeStart doubleValue]] name:startAddress POIId:nil];  //传入始点
    }
    if(endAddress&&latitudeEnd&&longitudeEnd){
      [config setRoutePlanPOIType:AMapNaviRoutePlanPOITypeEnd location:[AMapNaviPoint locationWithLatitude:[latitudeEnd doubleValue] longitude:[longitudeEnd doubleValue]] name:endAddress POIId:nil];  //传入终点
    }
    
    [self.compositeManager presentRoutePlanViewControllerWithOptions:config];
    
    result(@YES);
  } else {
    result(FlutterMethodNotImplemented);
  }
}
-(void)dismiss{
  if(self.mapViewController){
    [self.root dismissViewControllerAnimated:true completion:nil];
    self.mapViewController = nil;
  }
}


// init
- (AMapNaviCompositeManager *)compositeManager {
  if (!_compositeManager) {
    _compositeManager = [[AMapNaviCompositeManager alloc] init];  // 初始化
    _compositeManager.delegate = self;  // 如果需要使用AMapNaviCompositeManagerDelegate的相关回调（如自定义语音、获取实时位置等），需要设置delegate
  }
  return _compositeManager;
}



///**
// * @brief 单击地图回调，返回经纬度
// * @param mapView 地图View
// * @param coordinate 经纬度
// */
//- (void)mapView:(MAMapView *)mapView didSingleTappedAtCoordinate:(CLLocationCoordinate2D)coordinate{
//  NSLog(@"coordinate[%f,%f]",coordinate.latitude,coordinate.longitude);
//}

- (MAAnnotationView *)mapView:(MAMapView *)mapView viewForAnnotation:(id <MAAnnotation>)annotation
{
  if ([annotation isKindOfClass:[MAPointAnnotation class]])
  {
    static NSString *pointReuseIndentifier = @"pointReuseIndentifier";
    MAPinAnnotationView*annotationView = (MAPinAnnotationView*)[mapView dequeueReusableAnnotationViewWithIdentifier:pointReuseIndentifier];
    if (annotationView == nil)
    {
      annotationView = [[MAPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:pointReuseIndentifier];
    }
    annotationView.canShowCallout= YES;       //设置气泡可以弹出，默认为NO
    annotationView.animatesDrop = YES;        //设置标注动画显示，默认为NO
    annotationView.draggable = YES;        //设置标注可以拖动，默认为NO
    annotationView.pinColor = MAPinAnnotationColorPurple;
    return annotationView;
  }
  return nil;
}


-(void)initLocation:(BOOL)onceFlag{
  if(self.locationManager==nil){
    self.locationManager = [[AMapLocationManager alloc] init];
    self.locationManager.delegate = self;
  }
  [self.locationManager stopUpdatingLocation];
  if(onceFlag){
    // 带逆地理信息的一次定位（返回坐标和地址信息）
    [self.locationManager setDesiredAccuracy:kCLLocationAccuracyBest];
    //   定位超时时间，最低2s，此处设置为10s
    self.locationManager.locationTimeout =10;
    //   逆地理请求超时时间，最低2s，此处设置为10s
    self.locationManager.reGeocodeTimeout = 10;
    // 带逆地理（返回坐标和地址信息）。将下面代码中的 YES 改成 NO ，则不会返回地址信息。
    [self.locationManager requestLocationWithReGeocode:YES completionBlock:^(CLLocation *location, AMapLocationReGeocode *regeocode, NSError *error) {
      
      if (error)
      {
        NSLog(@"locError:{%ld - %@};", (long)error.code, error.localizedDescription);
        
        if (error.code == AMapLocationErrorLocateFailed)
        {
          return;
        }
      }
      
      NSLog(@"location:%@", location);
      //可在其中解析amapLocation获取相应内容。
      [self invokeMethod:location];
      if (regeocode)
      {
        NSLog(@"reGeocode:%@", regeocode);
      }
    }];
  }else{
  
    [self.locationManager setLocatingWithReGeocode:YES];
    //iOS 9（不包含iOS 9） 之前设置允许后台定位参数，保持不会被系统挂起
    [self.locationManager setPausesLocationUpdatesAutomatically:NO];
    
    //iOS 9（包含iOS 9）之后新特性：将允许出现这种场景，同一app中多个locationmanager：一些只能在前台定位，另一些可在后台定位，并可随时禁止其后台定位。
    if ([[[UIDevice currentDevice] systemVersion] floatValue] >= 9) {
      self.locationManager.allowsBackgroundLocationUpdates = YES;
    }
    //开始持续定位
    [self.locationManager startUpdatingLocation];
  }
}
- (void)amapLocationManager:(AMapLocationManager *)manager didUpdateLocation:(CLLocation *)location reGeocode:(AMapLocationReGeocode *)reGeocode
{
  NSLog(@"location:{lat:%f; lon:%f; accuracy:%f}", location.coordinate.latitude, location.coordinate.longitude, location.horizontalAccuracy);
  [self invokeMethod:location];
  if (reGeocode)
  {
    NSLog(@"reGeocode:%@", reGeocode);
  }
}
//调用flutter坐标更新接口
- (void)invokeMethod:(CLLocation *)location{
  double latitude = location.coordinate.latitude;
  double longitude = location.coordinate.longitude;
  NSMutableDictionary* dic = [[NSMutableDictionary alloc]init];
  [dic setObject:[NSString stringWithFormat:@"%f",latitude] forKey:@"latitude"];
  [dic setObject:[NSString stringWithFormat:@"%f",longitude] forKey:@"longitude"];
  if(self->_channel!=nil){
    [self->_channel invokeMethod:@"locationUpdate" arguments:dic];
  }
}
@end
