import 'dart:async';

import 'package:flutter/services.dart';

class EftFlutterAmap {
  static const MethodChannel _channel =
      const MethodChannel('eft_flutter_amap');
  static setMethodCallHandler(Future<dynamic> handler(MethodCall call)){
    _channel.setMethodCallHandler(handler);
}

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }
  ///设置IOS KEY,Android KEY需在AndroidManifest.xml中设置
  static Future<bool> initKey(String key) async {
    if(key==null||key.length==0){
      return false;
    }
    final bool result = await _channel.invokeMethod('initKey',key);
    return result;
  }
  ///调用更新定位接口,调用成功后MethodCallHandler中更新定位
  static Future<bool> initLocation(bool onceFlag) async {
    Map<String,String> map = new Map();
    map["onceFlag"] = onceFlag?"1":"0";

    final bool result = await _channel.invokeMethod('initLocation',map);
    return result;
  }
  ///停止定位
  static Future<bool> stopLocation() async {
    final bool result = await _channel.invokeMethod('stopLocation');
    return result;
  }
  ///销毁定位
  static Future<bool> destoryLocation() async {
    final bool result = await _channel.invokeMethod('destoryLocation');
    return result;
  }
  ///显示地图,默认当前位置
  static Future<bool> showMap(String title,String marker,EFTAmapLoaction location) async {
    Map<String,String> map = Map();
    if(location!=null){
      map["title"] = "$title";
      map["marker"] = "$marker";
      map["latitude"] = "${location.latitude}";
      map["longitude"] = "${location.longitude}";
    }
    final bool result = await _channel.invokeMethod('showMap',map);
    return result;
  }
  ///显示导航,从开始位置导航到结束位置
  static Future<bool> showNavMap({EFTAmapLoaction startLocation,EFTAmapLoaction endLocation,String startAddress,String endAddress}) async {
    Map<String,String> map = Map();
    if(startLocation!=null){
      map["latitudeStart"] = "${startLocation.latitude}";
      map["longitudeStart"] = "${startLocation.longitude}";
    }
    if(endLocation!=null){
      map["latitudeEnd"] = "${endLocation.latitude}";
      map["longitudeEnd"] = "${endLocation.longitude}";
    }
    if(startAddress!=null&&startAddress.length>0){
      map["startAddress"] = startAddress;
    }
    if(endAddress!=null&&endAddress.length>0){
      map["endAddress"] = endAddress;
    }

    final bool result = await _channel.invokeMethod('showNavMap',map);
    return result;
  }
}
class EFTAmapLoaction{
  EFTAmapLoaction(this.latitude,this.longitude);
  double latitude;
  double longitude;
}